
#pragma once

#include "NanoTrakStatus.h"

/**
* @namespace Thorlabs
* @brief The Thorlabs namespace for all Thorlabs software
*/
namespace Thorlabs {
	/**
	* @namespace Thorlabs::MotionControl
	* @brief The MotionControl namespace for all Thorlabs Motion Control software
	*/
	namespace MotionControl {
		/**
		* @namespace Thorlabs::MotionControl::GenericNanoTrakCLI
		* @brief The GenericNanoTrak CLI namespace for all common NanoTrak code written for DotNet in c++ / CLI
		*/
		namespace GenericNanoTrakCLI
		{
	using namespace System;

			/// <summary> NanoTrak Status. </summary>
			public ref class NanoTrakStatusBase
			{
			public:

				/// <summary> Constructor. </summary>
				NanoTrakStatusBase()
				{
					_status = 0;
					_valid = false;
				}

				/// <summary> Copy Constructor. </summary>
				/// <param name="status"> The copy constructor. </param>
				NanoTrakStatusBase(NanoTrakStatusBase ^status)
				{
					_status = status->_status;
					_valid = status->_valid;
				}

#pragma region internal_functions_and_declarations
				/// \cond INTERNAL_ONLY
				/// <summary> Constructor. </summary>
				/// <param name="status"> [in,out] The status. </param>
				NanoTrakStatusBase(std::shared_ptr<GenericNanoTrak::CNanoTrakStatusBaseStruct> status)
				{
					_status = status->Status();
					_valid = true;
				}

				/// \endcond
#pragma endregion

				/// <summary> Equality operator. </summary>
				/// <param name="status"> The status. </param>
				/// <returns> <c>true</c> if the parameters are considered equivalent. </returns>
				inline bool operator==(NanoTrakStatusBase^ status)
				{
					return ((status != (Object^)nullptr) && (status->_status == _status) && (status->_circlePosition == _circlePosition));
				}

				/// <summary> Inequality operator. </summary>
				/// <param name="status"> The status. </param>
				/// <returns> <c>true</c> if the parameters are not considered equivalent. </returns>
				inline bool operator!=(NanoTrakStatusBase^ status)
				{
					return ((status == (Object^)nullptr) || (status->_status != _status) || (status->_circlePosition != _circlePosition));
				}

				/// <summary> Gets the circle position. </summary>
				/// <value> The circle position as Horizontal and Vertical positions in the range 0 to 10 NT Units. </value>
				property HVPosition CirclePosition
				{ HVPosition get() { return _circlePosition; } }

				/// <summary> Gets the status bits. </summary>
				/// <returns>	The status bits from the device </returns>
				property unsigned int Status
				{ unsigned int get() { return _status; } }

				/// <summary> Determine whether the reading is valid. </summary>
				/// <value> <c>true</c> if the reading is valid. </value>
				property bool Valid
				{ bool get() { return _valid; } }

			private:
				/// <summary> The status. </summary>
				unsigned int _status;
				bool _valid;
			};


    
	/// <summary> Additional information for NanoTrak status events. </summary>
	/// <seealso cref="EventArgs"/>
	generic <typename T> where T : NanoTrakStatusBase
	public ref class NanoTrakStatusEventArgs : public EventArgs
	{
	private:
		/// <summary> The status. </summary>
		T _status;

	public:
		/// <summary> Constructor. </summary>
		/// <param name="status"> The <see cref="NanoTrakStatus">status</see>. </param>
		NanoTrakStatusEventArgs(T status)
		{
			_status = status;
		}

		/// <summary> Gets the status. </summary>
		/// <value> The <see cref="NanoTrakStatus">status</see>. </value>
		property T Status
		{ T get() { return _status; } }
	};

	/// <summary> NanoTrak error update delegate. </summary>
	/// <param name="sender"> The sender. </param>
	/// <param name="e">	  The ErrorUpdateEventArgs ^ to process. </param>
	generic <typename T> where T : NanoTrakStatusBase
	public delegate void NanoTrakStatusUpdateDelegate(Object ^sender, NanoTrakStatusEventArgs<T> ^e);


	/// <summary> Interface for class. </summary>
	/// <seealso cref="Thorlabs::MotionControl::DeviceManagerCLI::IGenericDeviceCLI"/>
	/// <seealso cref="Thorlabs::MotionControl::DeviceManagerCLI::IGenericCoreDeviceCLI"/>
	generic <typename T> where T : NanoTrakStatusBase
	public interface class IGenericNanoTrakBase : public IGenericDeviceCLI, public IGenericCoreDeviceCLI
	{
		/// <summary> Event source for all listeners interested in NanoTrak StatusUpdate events. </summary>
		event NanoTrakStatusUpdateDelegate<T>^ StatusUpdate;

		/// <summary> Gets the current NanoTrak status. </summary>
		/// <value> The current <see cref="NanoTrakStatus">NanoTrak status</see>. </value>
		/// <exception cref="Thorlabs::MotionControl::DeviceManagerCLI::DeviceNotReadyException"> Thown when device is not initialized or not connected. </exception>
		property T Status
		{
			virtual T get();
		}
		
		// plus other unrelated stuff

	};
}}}


